@@ -45,7 +45,7 @@ def upload_mlmodel(source_path: str, mlmodel_name: str, run_id: str = None, vers
     else:
         filenames = [os.path.basename(source_path)]
         full_paths = {os.path.basename(source_path): os.path.abspath(source_path) }
-        num_bytes = os.path.getsize(source_path)
+        num_bytes = [os.path.getsize(source_path)]
     args = (run_id, filenames, full_paths, num_bytes, mlmodel_name, short_description, version)
     return manager.add_event("add_mlmodel", args, async_method=False)
 
