@@ -11,6 +11,7 @@ from pyhectiqlab.watermark import Watermark
 from pyhectiqlab.artifacts import SharedArtifactsManager
 from pyhectiqlab.notebooks import get_notebook_file
 from pyhectiqlab.utils import list_all_files_in_dir
+from pyhectiqlab.decorators import write_method, action_method, beta, will_be_depreciated
 
 from pyhectiqlab.mlmodels import download_mlmodel as ops_download_mlmodel
 from pyhectiqlab.mlmodels import upload_mlmodel as ops_upload_mlmodel
@@ -19,7 +20,7 @@ from pyhectiqlab.datasets import upload_dataset as ops_upload_dataset
 from pyhectiqlab.mlmodels import mlmodel_info_at_path
 from pyhectiqlab.datasets import dataset_info_at_path
 
-from functools import partial, wraps
+from functools import partial
 from typing import Optional
 from sys import getsizeof
 import packaging 
@@ -104,31 +105,6 @@ class Run():
         self.logs_buffer = Buffer(push_method=self._push_logs)
         return
 
-    def write_method(f):
-        @wraps(f)
-        def wrapper(self, *args, **kwargs):
-            if self.read_only:
-                print('Run is read only. No actions.')
-                return
-            return f(self, *args, **kwargs)
-        return wrapper
-
-    def beta(f):
-        @wraps(f)
-        def wrapper(self, *args, **kwargs):
-            print('This method is still in development and can be unstable.')
-            return f(self, *args, **kwargs)
-        return wrapper
-
-    def will_be_depreciated(suggested_method):
-        def _method(f):
-            @wraps(f)
-            def wrapper(self, *args, **kwargs):
-                print(f'This method will be removed in a future release. Use {suggested_method} instead.')
-                return f(self, *args, **kwargs)
-            return wrapper
-        return _method
-
     def set_metrics_cache_settings(self, aggr: str = 'mean', max_cache_timeout=None, max_cache_length=None, min_cache_flush_delay=None):
         """
         Update the settings of the metrics manager.
@@ -151,12 +127,12 @@ class Run():
         self.events_manager = EventsManager(self._mock)
         self._read_only = True
 
-    @write_method
     @will_be_depreciated('add_artifact')
     def add_artifacts(self, *args, **kwargs):
         self.add_artifact(*args, **kwargs)
 
     @write_method
+    @action_method
     def add_artifact(self, filepath: str, step: int = None, wait_response: bool = False):
         """Log a file as an artifacts. If a file already exists with the name, it will be overwritten.
 
@@ -170,6 +146,7 @@ class Run():
         return self.events_manager.add_event("add_artifact", args, async_method=bool(1-wait_response))
 
     @write_method
+    @action_method
     def add_mlmodel_usage(self, mlmodel_name: str, version: str = None):
         """Manually log that the run is using a specific model. You don't need
         to use this method if you already use `run.download_mlmodel`.
@@ -181,6 +158,7 @@ class Run():
         self.events_manager.add_event("log_mlmodel", args)
 
     @write_method
+    @action_method
     def add_mlmodel_usage_from_dirpath(self, dirpath:str):
         """Manually log that the run is using a specific model, but where 
         the name and the version are extracted from the path of the mlmodel. You don't need
@@ -196,7 +174,8 @@ class Run():
         self.events_manager.add_event("log_mlmodel", args)
         return
         
-    @write_method        
+    @write_method
+    @action_method        
     def add_mlmodel(self, source_path: str, name: str, version: str = None, short_description: str = None,
                 push_dir: bool = False):
         """Add a mlmodel.
@@ -218,6 +197,7 @@ class Run():
         if mlmodel is not None:
             self.add_mlmodel_usage(mlmodel.get('name'), version=mlmodel.get('version'))
 
+    @action_method
     def download_mlmodel(self, mlmodel_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing mlmodel from the run's project.
 
@@ -237,6 +217,7 @@ class Run():
         return dirpath
         
     @write_method
+    @action_method
     def add_dataset(self, source_path: str, name: str, version: str = None, short_description: str = None,
                 push_dir: bool = False):
         """Add a dataset.
@@ -259,6 +240,7 @@ class Run():
             self.add_dataset_usage(dataset.get('name'), version=dataset.get('version'))
 
     @write_method
+    @action_method
     def add_dataset_usage(self, dataset_name: str, version: str = None):
         """Manually log that the run is using a specific dataset You don't need
         to use this method if you already use `run.download_dataset`.
@@ -271,6 +253,7 @@ class Run():
         self.events_manager.add_event("log_dataset", args)
 
     @write_method
+    @action_method
     def add_dataset_usage_from_dirpath(self, dirpath:str):
         """Manually log that the run is using a specific dataset. You don't need
         to use this method if you already use `run.download_dataset`.
@@ -285,6 +268,7 @@ class Run():
         self.events_manager.add_event("log_dataset", args)
         return
 
+    @action_method
     def download_dataset(self, dataset_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing dataset from the run's project.
 
@@ -304,6 +288,7 @@ class Run():
         return dirpath
     
     @write_method
+    @action_method
     def add_config(self, config: 'Config', prefix=""):
         """Log a config object (pyhectiqlab.Config). Use prefix 
         to log a config in a nested path.
@@ -311,6 +296,7 @@ class Run():
         config.push_as_meta(self, prefix=os.path.join("config/", prefix))
 
     @write_method
+    @action_method
     @will_be_depreciated('add_tf_model_as_artifact')
     def add_tf_model(self, *args, **kwargs):
         """Add a tensorflow model as an artifact. This method will be depreciated. Use 
@@ -319,6 +305,7 @@ class Run():
         self.add_tf_model_as_artifact(*args, **kwargs)
 
     @write_method
+    @action_method
     def add_tf_model_as_artifact(self, model: 'tf.keras.Model', filename: str, step: int = None):
         """Add a tensorflow model as an artifact.
         """
@@ -329,6 +316,7 @@ class Run():
         self.add_artifact(f"{p}/{filename}.zip", step=step)
 
     @write_method
+    @action_method
     def add_directory_as_zip_artifact(self, dirpath: str, step: int = None):
         """Compress a full directory and add the compressed file as an artifact.
         """
@@ -338,6 +326,7 @@ class Run():
         self.add_artifact(f"{p}/{filename}.zip", step=step)
 
     @write_method
+    @action_method
     def add_shared_artifact(self, filename: str):
         """Push a shared artifact.
         """
@@ -351,6 +340,7 @@ class Run():
         return m.list_artifacts(project_id=self.project_view["id"])
 
     @write_method
+    @action_method
     def add_metrics(self, key: 'str', value: float, step: int):
         """Add a metrics. The metrics is saved even if there already exists a value
         for the same key and step. 
@@ -363,6 +353,7 @@ class Run():
         self.metrics_manager.add(key, float(value), int(step))
 
     @write_method
+    @action_method
     def add_tag(self, name: str, description: str = None, color: str = None):
         """Add tag to a run. If the tag does not already exists by this name,
         a new tag is created with the given description and color.
@@ -371,6 +362,7 @@ class Run():
         self.events_manager.add_event("add_tag", args)
 
     @write_method
+    @action_method
     def add_package_versions(self, packages: Optional[dict] =None, with_sys: bool = True, with_python: bool = True, with_git: bool = True):
         """Save the version of the imported packages.
 
@@ -394,6 +386,7 @@ class Run():
         self.events_manager.add_event("push_package_versions", args)
 
     @write_method
+    @action_method
     def add_package_repo_state(self, package: str = None):
         """Save the branch/commit/origin of the git repo in which
         a package is located.
@@ -408,6 +401,7 @@ class Run():
         self.events_manager.add_event("push_git_package_state", args)
 
     @write_method
+    @action_method
     def add_current_notebook(self, stamp: str = 'datetime'):
         """If the run is executed in a jupyter notebook, this
         will add your current notebook as an artifact.
@@ -419,6 +413,7 @@ class Run():
         self.add_artifact(filename)
 
     @write_method
+    @action_method
     def failed(self):
         """Set the run to the failed status.
         """
@@ -426,6 +421,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
 
     @write_method
+    @action_method
     def _stopped(self):
         """Set the run to the stopped status.
         """
@@ -433,6 +429,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
+    @action_method
     def completed(self):
         """Set the run to the completed with success status.
         """
@@ -440,6 +437,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
+    @action_method
     def pending(self):
         """Set the run to the pending status.
         """
@@ -447,6 +445,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
+    @action_method
     def running(self):
         """Set the run to the running status.
         """
@@ -454,6 +453,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
+    @action_method
     def training(self):
         """Set the run to the training status.
         """
@@ -461,6 +461,7 @@ class Run():
         self.events_manager.add_event("set_run_status", args)
         
     @write_method
+    @action_method
     def add_meta(self, key, value):
         """Add or update a meta information in the format of
         key/value.
@@ -469,6 +470,7 @@ class Run():
         self.events_manager.add_event("push_meta", args)
         
     @write_method
+    @action_method
     def set_note(self, text: str):
         """Add or update a custom note field for the run.
         """
@@ -476,6 +478,7 @@ class Run():
         self.events_manager.add_event("set_note", args)
     
     @write_method
+    @action_method
     def set_paper(self, content: str):
         """Overwrite the content of the paper of the run.
         """
@@ -483,6 +486,7 @@ class Run():
         self.events_manager.add_event("set_paper", args)
 
     @write_method
+    @action_method
     def start_recording_logs(self, logger_name=None):
         """Start tailing the logs on a logger name.
         If logger_name is None, the default __name__ will be taken and
@@ -528,6 +532,7 @@ class Run():
             return logging.getLogger(self.logger_name)
         
     @write_method
+    @action_method
     def stop_recording_logs(self):
         """Stop recording the logs from the logger. 
         """
@@ -535,6 +540,7 @@ class Run():
         self.logs_buffer.stop(key="logs")
         
     @write_method
+    @action_method
     def _push_logs(self, key, msg):
         if key=="logs":
             if self.is_recording_logs:
@@ -542,6 +548,7 @@ class Run():
                 self.events_manager.add_event("append_logs", args)
         
     @write_method
+    @action_method
     def _push_metrics(self, key, values):
         args = (self._id, key, values)
         self.events_manager.add_event("push_metrics", args)
